--[[
    Baldrick by LordFarlander
    Version 1.6.137
]]--

--[[
Copyright (c) 2008-2012, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 137 $
-- $Date: 2012-04-09 01:56:49 +0000 (Mon, 09 Apr 2012) $

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Baldrick" )

_G["BINDING_NAME_CLICK Baldrick_Button:LeftButton"] = L["Use Quest Item"]
BINDING_HEADER_BALDRICKHEADER = L["Baldrick"]

local LibLordFarlander = LibStub( "LibLordFarlander-2.0", true )
if( not LibLordFarlander ) then
    error( L["%s requires %s"]:format( L["Baldrick"], "LibLordFarlander-2.0" ) )
    return
end--if

local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true )
if( not TR ) then
    error( L["%s requires %s"]:format( L["Baldrick"], "LibLordFarlander-TableRecycler-1.0" ) )
    return
end--if

local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.1", true )
if( not LibLordFarlander_UI ) then
    error( L["%s requires %s"]:format( L["Baldrick"], "LibLordFarlander-UI-2.1" ) )
    return
end--if

local quixote = LibStub( "LibQuixote-2.0", true )
if( not quixote ) then
    error( L["%s requires %s"]:format( L["Baldrick"], "LibQuixote-2.0" ) )
    return
end--if

Baldrick = LibStub( "AceAddon-3.0" ):NewAddon( "Baldrick", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0", "LibLordFarlander-UI-2.1" )
Baldrick.version = { major = 1, minor = 6, revision = tonumber( ("$Revision: 137 $"):match( "(%d+)" ) ) }
Baldrick.date = ("$Date: 2012-04-09 01:56:49 +0000 (Mon, 09 Apr 2012) $"):sub( 8, 17 )

Baldrick.DEBUG = false            --true = Debugging Mode

Baldrick.dprint = LibLordFarlander.dprint
Baldrick.button = nil
Baldrick.icon = [[Interface\AddOns\Baldrick\img\Baldrick]]

Baldrick.Dirty = false
Baldrick.DirtyBags = {}

Baldrick.EventRegistry = TR:GetTable(
    --Prevent these from being overridden
    "Quest_Gained", true,
    "Quest_Failed", true,
    "Quest_Complete", true,
    "Quest_Lost", true,
    "BAG_UPDATE", true,
    "PLAYER_REGEN_ENABLED", true,
    "BAG_UPDATE_COOLDOWN", true,
    "UNIT_INVENTORY_CHANGED", true )

Baldrick.RegisteredCallbackLibraries = TR:GetTable()

Baldrick.QuestModules = TR:GetTable()
Baldrick.AllQuests = TR:GetTable()

function Baldrick:OnInitialize()
    local AceConfigReg = LibStub( "AceConfigRegistry-3.0" )
    local AceConfigDialog = LibStub( "AceConfigDialog-3.0" )
    local defaults = {
        profile = {
            Modules = {},
            Button = LibLordFarlander_UI.GetButtonSettingsDefaults(),
        }
    }
    local buttonOptions = self:GetButtonOptionGroup( {
            name = L["Button"],
            desc = L["Options relating to the button."],
            yes = L["yes"],
            no = L["no"],
            show = L["showing"],
            hide = L["hiding"],
            locked = L["locked"],
            unlocked = L["unlocked"],
            toggleButton = L["toggleButton"],
            toggleButton_name = L["Show button"],
            toggleButton_desc = L["Show/hide the button."],
            autoHideButton = L["autoHideButton"],
            autoHideButton_name = L["Automatically hide button"],
            autoHideButton_desc = L["Toggle the autohiding of the button when there is nothing to do."],
            lockButton = L["lockButton"],
            lockButton_name = L["Lock button position"],
            lockButton_desc = L["Lock/unlock the position of the button."],
            alpha = L["alpha"],
            alpha_name = L["Alpha of button"],
            alpha_desc = L["Sets the alpha value that the button has normally."],
            fadeOut = L["fadeOut"],
            fadeOut_name = L["Fade out"],
            fadeOut_desc = L["Fade the button out when the cursor is not over it?"],
            fadeOutInsteadHide = L["fadeOutInsteadHide"],
            fadeOutInsteadHide_name = L["Fade out instead of hiding"],
            fadeOutInsteadHide_desc = L["Fade the button out instead of hiding it?"],
            fadeOutCancelInCombat = L["fadeOutCancelInCombat"],
            fadeOutCancelInCombat_name = L["Cancel fade out if in combat"],
            fadeOutCancelInCombat_desc = L["Should the button not fade out if you're in combat?"],
            fadeOutCancelOnShift = L["fadeOutCancelOnShift"],
            fadeOutCancelOnShift_name = L["Cancel fade out if holding Shift"],
            fadeOutCancelOnShift_desc = L["Should the button not fade out if you're holding Shift?"],
            fadeOutCancelOnCtrl = L["fadeOutCancelOnCtrl"],
            fadeOutCancelOnCtrl_name = L["Cancel fade out if holding Ctrl"],
            fadeOutCancelOnCtrl_desc = L["Should the button not fade out if you're holding Ctrl?"],
            fadeOutCancelOnAlt = L["fadeOutCancelOnAlt"],
            fadeOutCancelOnAlt_name = L["Cancel fade out if holding Alt"],
            fadeOutCancelOnAlt_desc = L["Should the button not fade out if you're holding Alt?"],
            fadeOutTime = L["fadeOutTime"],
            fadeOutTime_name = L["Fade out time"],
            fadeOutTime_desc = L["Sets how long the button takes to fade out."],
            fadeOutDelay = L["fadeOutDelay"],
            fadeOutDelay_name = L["Fade out delay"],
            fadeOutDelay_desc = L["Sets the delay before the button starts to fade."],
            fadeOutAlpha = L["fadeOutAlpha"],
            fadeOutAlpha_name = L["Alpha after fadeout"],
            fadeOutAlpha_desc = L["Sets the alpha value that the button has after faded out."],
            buttonScale = L["buttonScale"],
            buttonScale_name = L["Scale of button"],
            buttonScale_desc = L["Sets the scaling value for the button to make it bigger or smaller."],
        }, 1 )
    local options = {
        type = "group",
        handler = self,
        args = {
            modules = {
                type = "multiselect",
                order = 1,
                name = L["Modules"],
                desc = L["Selects which modules to load."],
                width = "full",
                cmdHidden = true,
                values = function( info )
                             local Modules = {}

                             for name, module in pairs( info.handler.QuestModules ) do
                                 if( ( not module.checkLoad ) or module.checkLoad() ) then
                                     Modules[name] = name
                                 end--if
                             end--for
                             return( Modules )
                         end,
                get = function( info, key )
                          return info.handler.db.profile.Modules[key]
                      end,
                set = function( info, key, val )
                          if( val == nil ) then
                              val = false
                          end--if
                          info.handler.db.profile.Modules[key] = val
                          info.handler:RebuildQuestLists()
                      end,
            },
        },
        name = L["Baldrick"],
        desc = L["Addon options."],
        icon = self.icon,
    }

    self.db = LibStub( "AceDB-3.0" ):New( "BaldrickDB", defaults )
    self.db.RegisterCallback( self, "OnProfileChanged" )
    self.db.RegisterCallback( self, "OnProfileReset" )

    local profileOptions = LibStub( "AceDBOptions-3.0" ):GetOptionsTable( self.db )
    local slashOptions = {
        type = "group",
        handler = self,
        name = options.name,
        desc = options.desc,
        icon = options.icon,
        args = {
            [L["button"]] = buttonOptions,
            [L["profile"]] = profileOptions,
            [L["gui"]] = {
                type = "execute",
                order = -1,
                name = L["Open GUI configurator"],
                desc = L["Open GUI configurator"],
                guiHidden = true,
                func = function( info )
                           InterfaceOptionsFrame_OpenToCategory( L[info.handler.name] )
                       end,
            },
            debug = {
                type = "toggle",
                hidden = true,
                name = "Debug",
                desc = "Debug mode toggle",
                desc = function( info )
                            local returnString = "Debug mode toggle"

                            if( info.uiType == "cmd" ) then
                                returnString = ("%s |cFFFF0000[%s]|r"):format( returnString, info.handler.DEBUG and L["yes"] or L["no"] )
                            end--if
                           return returnString
                       end,
                get = function( info )
                          return info.handler.DEBUG
                      end,
                set = function( info, val )
                          info.handler.DEBUG = val
                          info.handler:Print( ("debug mode |cFFFF0000[%s]|r"):format( info.handler.DEBUG and L["yes"] or L["no"] ) )
                      end,
            },
        },
    }

    LibStub( "AceConfig-3.0" ):RegisterOptionsTable( L["Baldrick"], slashOptions, { L["baldrick"], L["br"] } )
    AceConfigReg:RegisterOptionsTable( "Baldrick Main Options", options )
    AceConfigReg:RegisterOptionsTable( "Baldrick Button", buttonOptions )
    AceConfigReg:RegisterOptionsTable( "Baldrick Profile", profileOptions )
    AceConfigDialog:AddToBlizOptions( "Baldrick Main Options", L["Baldrick"] )
    AceConfigDialog:AddToBlizOptions( "Baldrick Button", L["Button"], L["Baldrick"] )
    AceConfigDialog:AddToBlizOptions( "Baldrick Profile", L["Profile"], L["Baldrick"] )

    if( LibStub( "LibDataBroker-1.1", true ) ) then
        self.ldbobj = LibStub( "LibDataBroker-1.1" ):NewDataObject( L["Baldrick"], {
                type = "data source",
                icon = self.icon,
                label = L["Baldrick"],
                text = L["Nothing to do m'lord..."],
                secureTemplate = "SecureActionButtonTemplate",
                OnClick = function( self, btn )
                              if( self:IsObjectType( "SecureActionButtonTemplate" ) ) then
                                  Baldrick.button:Click( btn )
                              end--if
                          end,
                OnTooltipShow = function( tooltip )
                                    if( not tooltip ) then
                                        return
                                    end--if
                                    Baldrick:SetTooltip( tooltip )
                                end,
            } )
    end--if
end--Baldrick:OnInitialize()

function Baldrick:AutoHideChange()
    self:SetButton( self:GetItemToShow( self.AllQuests ) )
end--Baldrick:AutoHideChange()

function Baldrick:OnProfileReset()
    self:OnProfileChanged()
end--Baldrick:OnProfileReset()

function Baldrick:OnProfileChanged()
    self:OnProfileChanged_UI()
    self.db.profile.LastWoWVersion = LibLordFarlander.GetTOCVersion()
    self.db.profile.LastVersion = self.version
    self:BuildQuestLists()
end--Baldrick:OnProfileChanged()

function Baldrick:OnEnable()
    --Register the Events
    quixote.RegisterCallback( self, "Quest_Gained", "Update" )
    quixote.RegisterCallback( self, "Quest_Complete", "Update" )
    quixote.RegisterCallback( self, "Quest_Failed", "Update" )
    quixote.RegisterCallback( self, "Quest_Lost", "Update" )
    quixote.RegisterCallback( self, "Objective_Update", "Update" )

    self:RegisterEvent( "BAG_UPDATE",
        function( _, bagID )
            self.Dirty = true
            if( bagID ) then
                self.DirtyBags[bagID] = true
            end--if
            if( not self.button:IsVisible() ) then
                self:ScheduleTimer( "OnUpdate", 0 )
            end--if
        end )
    self:RegisterEvent( "PLAYER_REGEN_ENABLED",
        function()
            self.button:OutOfCombatReset()
            self:Update( "PLAYER_REGEN_ENABLED" )
        end )

    self:RegisterEvent( "BAG_UPDATE_COOLDOWN",
        function( ... )
            self:UpdateCooldown()
            self:Update( ... )
        end )
    self:RegisterEvent( "UNIT_INVENTORY_CHANGED",
        function( _, unitID )
            if( unitID == "player" ) then
                self:UpdateCooldown()
                self:Update( "UNIT_INVENTORY_CHANGED", unitID )
            end--if
        end )

    if( not self.button ) then
        self:SetupForUI()
        self.button = self:CreateButton( self.name, self.icon )
        self.button:SetMovable( not self.db.profile.Button.LockButton )
    end--if

    self.lastItem = nil

    self:OnProfileChanged()
end--Baldrick:OnEnable()

function Baldrick:OnDisable()
    quixote.UnregisterAll( self )
    for library, _ in pairs( self.RegisteredCallbackLibraries ) do
        LibStub( library ).UnregisterAll( self )
    end--for
    TR:ClearTable( self.RegisteredCallbackLibraries )
    TR:ClearTable( self.AllQuests )
end--Baldrick:OnDisable()

function Baldrick:OnUpdate( elapsed )
    if( self.Dirty ) then
        --Update the quest item if needed
        self.Dirty = false
        self:Update( "BAG_UPDATE", self.DirtyBags )
        table.wipe( self.DirtyBags )
    end--if
end--Baldrick:OnUpdate()

function Baldrick:Update( event, ... )
    local questList = self.AllQuests
    local eventRegistry = self.EventRegistry[event]

    if( ( type( eventRegistry ) == "table" ) and TR.HasTableMembers( eventRegistry ) ) then
        if( self.DEBUG ) then
            self:Print( ("Checking for event %s"):format( event ) )
        end--if
        questList = eventRegistry
    end--if
    self:SetButton( self:GetItemToShow( questList, event, ... ) or self.lastItem )
end--Baldrick:Update()

function Baldrick:TimerUpdate()
    self:Update( "!COORDTIMER" )
end--Baldrick:TimerUpdate()

----------------------------
--+Quest Functions--
function Baldrick:RebuildQuestLists()
    -- Set up lists of certain events so that related quests are only checked on those events
    TR:ClearTable( self.AllQuests )
    TR:ClearTable( self.EventRegistry )
    --Prevent these from being overridden
    self.EventRegistry["Quest_Gained"] = true
    self.EventRegistry["Quest_Failed"] = true
    self.EventRegistry["Quest_Complete"] = true
    self.EventRegistry["Quest_Lost"] = true
    self.EventRegistry["Objective_Update"] = true
    self.EventRegistry["BAG_UPDATE"] = true
    self.EventRegistry["PLAYER_REGEN_ENABLED"] = true
    self.EventRegistry["BAG_UPDATE_COOLDOWN"] = true
    self.EventRegistry["UNIT_INVENTORY_CHANGED"] = true
    self:BuildQuestLists()
end--Baldrick:RebuildQuestLists()

function Baldrick:BuildQuestLists()
    -- Set up lists of certain events so that related quests are only checked on those events
    local Modules = self.db.profile.Modules

    for name, module in pairs( self.QuestModules ) do
        if( type( Modules[name] ) == "nil" ) then
            Modules[name] = true
        end--if
        if( Modules[name] and ( ( not module.checkLoad ) or module.checkLoad() ) ) then
            if( self.DEBUG ) then
                self:Print( ("Registering module %s"):format( name ) )
            end--if
            for _, quest in pairs( module.quests ) do
                self:RegisterQuest( quest )
            end--for
        end--if
    end--for
end--Baldrick:BuildQuestLists()

function Baldrick:RegisterQuestModule( moduleName, questList, checkLoadFunc )
    if( not self.QuestModules[moduleName] ) then
        self.QuestModules[moduleName] = TR:GetTable( "quests", questList, "checkLoad", checkLoadFunc )
        if( self.button ) then
            self:BuildQuestLists()
        end--if
    end--if
end--Baldrick:RegisterQuestModule( moduleName, questList, noUseFunc )

local function InsertQuestInList( list, quest )
    for index, cquest in ipairs( list ) do
        --We want quests with targets first
        --Then quests with zones
        --Lastly the rest
        if( ( quest.targets and ( not cquest.targets ) ) or
            ( ( ( quest.zones ) and ( not cquest.zones ) ) and ( not cquest.targets ) ) ) then
            table.insert( list, index, quest )
            return
        end--if
    end--for
    table.insert( list, quest )
end--InsertQuestInList( list, quest )

function Baldrick:RegisterQuest( quest )
    if( self.DEBUG ) then
        self:dprint( ("Registering quest %i"):format( ( type( quest.questIDs ) == "table" ) and quest.questIDs[1] or quest.questIDs ) )
    end--if
    if( ( type( quest.subZones ) ~= "nil" ) or ( type( quest.zones ) ~= "nil" ) ) then
        self:RegisterQuestEvent( "ZONE_CHANGED", quest )
        self:RegisterQuestEvent( "ZONE_CHANGED_NEW_AREA", quest )
    end--if
    if( ( type( quest.target ) ~= "nil" ) or ( type( quest.targets ) ~= "nil" ) ) then
        if( not quest.norange ) then
            self:RegisterQuestEvent( "!COORDTIMER", quest )
        end--if
        self:RegisterQuestEvent( "PLAYER_TARGET_CHANGED", quest )
        self:RegisterQuestEvent( "UNIT_NAME_UPDATE", quest,
            function( event, unitid, ... )
                if( ( unitid == "playertarget" ) or ( unitid == "target" ) ) then
                    self:Update( event, unitid, ... )
                end--if
            end )
    end--if
    if( type( quest.additionalEvents ) == "table" ) then
        for _, additionalEvent in pairs( quest.additionalEvents ) do
            self:RegisterQuestEvent( additionalEvent, quest )
        end--for
    elseif( type( quest.additionalEvents ) == "string" ) then
        self:RegisterQuestEvent( quest.additionalEvents, quest )
    end--if
    InsertQuestInList( self.AllQuests, quest )
end--Baldrick:AddQuest()

function Baldrick:RegisterQuestEvent( event, quest, functionCall )
    local libraryName = nil

    if( event:find( ":" ) ) then
        libraryName, event = strsplit( ":", event )
    end--if
    if( type( self.EventRegistry[event] ) == "nil" ) then
        if( self.DEBUG ) then
            self:Print( ("Registering event %s"):format( event ) )
        end--if
        if( type( functionCall ) == "nil" ) then
            functionCall = function( ... ) self:Update( ... ); end
        end--if
        self.EventRegistry[event] = TR:GetTable()
        if( libraryName ) then
            local library = LibStub( libraryName, true )

            if( library ) then
                library.RegisterCallback( self, event, functionCall )
                self.RegisteredCallbackLibraries[libraryName] = true
            end--if
        elseif( event == "!COORDTIMER" ) then
            self.timerHandle = self:ScheduleRepeatingTimer( self.TimerUpdate, 0.1, self )
        else
            self:RegisterEvent( event, functionCall )
        end--if
    end--if
    if( type( self.EventRegistry[event] ) == "table" ) then
        InsertQuestInList( self.EventRegistry[event], quest )
    end--if
end--Baldrick:RegisterQuest( event, quest )

function Baldrick:CheckQuestIDs( questIDs )
    if( type( questIDs ) == "number" ) then
        if( self.HasQuestAndNotComplete( questIDs ) ) then
            return questIDs
        end--if
    elseif( type( questIDs ) == "table" ) then
        for _, questID in pairs( questIDs ) do
            if( self.HasQuestAndNotComplete( questID ) ) then
                return questID
            end--if
        end--for
    end--if
    return nil
end--Baldrick.CheckQuestIDs( questIDs )

local function EscapeStringForFind( str )
    --gsub itself uses pattern matching, so the first param has to be escaped itself
    --This is not all of them, only the most likely
    return str:gsub( "%%", "%%" ):gsub( "%-", "%-" ):gsub( "%(", "%(" ):gsub( "%)", "%)" )
end--CleanStringForFind( str )

local function CheckItems( questItems )
    local bItems = true

    for _, item in pairs( questItems ) do
        if( item.questItem ) then
            bItems = bItems and ( LibLordFarlander.PlayerItemCount( item.questItem ) >= ( item.questItemCount or 1 ) )
        end--if
    end--for
    return bItems
end--CheckItems( questItems )

local function CheckZones( zones, zone )
    if( type( zones ) == "string" ) then
        local zoneTest = EscapeStringForFind( zone:lower() )
        local zonesTest = EscapeStringForFind( zones:lower() )

        if( ( zoneTest ~= "" ) and ( zoneTest:find( zonesTest, 1, true ) or zonesTest:find( zoneTest, 1, true ) ) ) then
            return zone
        end--if
    elseif( type( zones ) == "table" ) then
        for _, questZone in ipairs( zones ) do
            if( CheckSubZones( questZone, zone ) ) then
                return zone
            end--if
        end--for
        return nil
    end--if
    return type( zones ) == "nil"
end--CheckZones( zones, zone )

local function CheckTarget( targets, target )
    if( type( targets ) == "number" ) then
        if( Baldrick.DEBUG ) then
            Baldrick:Print( ("Target (%s) NPC ID: %05x"):format( target, tonumber( target:sub( -12, -9 ), 16 ) ) )
        end--if

        local guid = ("0xF13%05X"):format( targets ) --Only want to check the ID
        
        if( Baldrick.DEBUG ) then
            Baldrick:Print( ("Target check %s = %s"):format( target:sub( 1, guid:len(), guid ) ) )
        end--if

        if( target:sub( 1, guid:len() ) == guid ) then
            return targets
        end--if
    elseif( type( targets ) == "table" ) then
        for _, questTarget in ipairs( targets ) do
            if( CheckTarget( questTarget, target ) ) then
                return questTarget
            end--if
        end--for
        return nil
    end--if
    return type( targets ) == "nil"
end--CheckTarget( targets, target )

function Baldrick:GetItemToShow( questList, event, ... )
    local returnVal = nil

    if( questList ) then
        local zone = GetRealZoneText()
        local subZone = GetSubZoneText()
        local playerLevel = UnitLevel( "player" )
        local target = UnitGUID( "playertarget" )
        local eventFunctionName = event and ("On%s"):format( event )
        --function holders
        local CheckQuestIDs = self.CheckQuestIDs

        for _, quest in ipairs( questList ) do
            local eventFunction = event and quest[eventFunctionName]
            local useit = false
            local questItems = quest.questItems
            local default = questItems.default.questItem or questItems.default.questAction

            if( default and ( not returnVal ) ) then
                local questID = CheckQuestIDs( self, quest.questIDs )

                if( ( type( questID ) ~= "nil" ) and ( ( not quest.levelMin ) or ( quest.levelMin <= playerLevel ) ) ) then
                    local questTarget = ( type( target ) == "nil" ) or CheckTarget( quest.target or quest.targets, target )
                    local questSubZone = CheckZones( quest.subZones, subZone )
                    local questZone = CheckZones( quest.zones, zone )
                    local questFunc = quest.func

                    returnVal = ( CheckItems( questItems ) and
                                --The explicit check for true on questTarget is intentional
                                  ( ( questTarget == true ) or
                                    ( questTarget and
                                      ( quest.norange or
                                        ( ( ItemHasRange( default ) == nil ) or ( IsItemInRange( default, "playertarget" ) == 1 ) ) ) ) ) and
                                  ( questZone and questSubZone and
                                    ( ( type( questFunc ) ~= "function" ) or questFunc( quest, questID, questTarget, questSubZone ) ) ) ) and
                                default
                    useit = returnVal
                end--if
            end--if
            if( ( not useit ) and ( default == self.lastItem ) ) then
                self.lastItem = nil
            end--if
            if( type( eventFunction ) == "string" ) then
                quest[eventFunction]( quest, ... )
            elseif( type( eventFunction ) == "function" ) then
                eventFunction( quest, ... )
            end--if
        end--for
    end--if
    return returnVal or self.lastItem
end--Baldrick:GetItemToShow( questList, event, ... )
---Quest Functions--
----------------------------

-------------------
--+Help Functions--
function Baldrick.HasQuestAndNotComplete( questId )
    if( type( questId ) ~= "number" ) then
        questId, _, _, _, _, _, _, _, _, _  = quixote:GetQuest( questId )
    end--if
    if( type( questId ) == "number" ) then
        local uid, _, _, _, _, _, complete, _, _, _ = quixote:GetQuestByUid( questId )

        if( ( type( uid ) ~= "nil" ) and ( not complete ) ) then
            if( Baldrick.DEBUG ) then
                Baldrick:Print( ("Player is on quest %s"):format( questId ) )
            end--if
        end--if
        return ( type( uid ) ~= "nil" ) and ( not complete )
    end--if
    return false
end--self.HasQuestAndNotComplete( questId )

function Baldrick.QuestObjectiveComplete( questId, objective )
    if( type( questId ) ~= "number" ) then
        questId, _, _, _, _, _, _, _, _, _  = quixote:GetQuest( questId )
    end--if
    if( type( questId ) == "number" ) then
        if( type( objective ) == "string" ) then
            local _, numItems, numNeeded = quixote:GetQuestObjective( questId, objective )

            return numItems == numNeeded
        elseif( type( objective ) == "number" ) then
            -- objective "position"
            local desc, _, done = GetQuestLogLeaderBoard( objective, questId )

            return ( desc and done ) ~= nil
        end--if
    end--if
    return false
end--Baldrick.QuestObjectiveComplete( questId, objective )

function Baldrick.GetQuestName( questId )
    return select( 3, quixote:GetQuestByUid( questId ) )
end--Baldrick.GetQuestName( questId )
---Help Functions--
-------------------

-----------------
--+UI Functions--
function Baldrick:SetTooltip( tooltip )
    if( type( self.lastItem ) == "nil" ) then
        tooltip:SetText( L["Nothing to do m'lord..."] )
    else
        tooltip:SetHyperlink( LibLordFarlander.GetItemLink( self.lastItem ) )
    end--if
end--Baldrick:SetTooltip( tooltip )

function Baldrick:SetGameTooltips()
    local dqButton = self.button

    if( not self.keyBoundMode ) then
        local dqTooltip = dqButton.tooltip

        dqTooltip:SetOwner( self.button, "RIGHT" )
        self:SetTooltip( dqTooltip )
    end--if
    LibLordFarlander_UI.AfterSetTooltip( dqButton )
end--Baldrick:SetGameTooltips()

function Baldrick:UpdateCooldown()
    local itemLink = self.lastItem

    if( itemLink ) then
        local startTime, duration, enable = GetItemCooldown( itemLink )
        local cooldownInicator = self.button.cooldown

        if( startTime and duration and enable and ( startTime > 0 ) and ( duration > 0 ) ) then
            cooldownInicator:Show()
            cooldownInicator:SetCooldown( startTime, duration )
        else
            cooldownInicator:Hide()
        end--if
    end--if
end--Baldrick:UpdateCooldown()

function Baldrick:SetButton( questItem )
    if( not InCombatLockdown() ) then
        local dqButton = self.button
        local cooldownInicator = dqButton.cooldown
        local lastitem = self.lastItem
        local chardb = self.db.profile
        local ldbobj = self.ldbobj

        -- ONLY change if not in combat
        if( type( questItem ) == "nil" ) then
            cooldownInicator:Hide()
            dqButton:SetAttribute( "*type1", "item" )
            dqButton:SetAttribute( "*item1", nil )
            dqButton:SetImage( self.icon )

            ldbobj.icon = self.icon
            ldbobj.text = L["Nothing to do m'lord..."]
            if( chardb.Button.AutoHideButton ) then
                dqButton:HideEx()
            end--if
        elseif( type( questItem ) == "number" ) then
            local sName, _, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( questItem )
            local startTime, duration, enable = GetItemCooldown( questItem )

            dqButton:SetAttribute( "*type1", "item" )
            dqButton:SetAttribute( "*item1", sName )
            dqButton:SetImage( sItemTexture )

            ldbobj.icon = sItemTexture
            ldbobj.text = sName

            dqButton:Enable()
            if( chardb.Button.AutoHideButton ) then
                dqButton:ShowEx()
            end--if
            if( ( lastitem ~= questItem ) and startTime and duration and enable and ( startTime > 0 ) and ( duration > 0 ) ) then
                cooldownInicator:Show()
                cooldownInicator:SetCooldown( startTime, duration )
            end--if
        elseif( ( type( questItem ) == "table" ) and questItem.macro ) then
            cooldownInicator:Hide()
            dqButton:SetAttribute( "*type1", "macro" )
            dqButton:SetAttribute( "macrotext", questItem.macro )
            dqButton:SetImage( questItem.icon )

            ldbobj.icon = questItem.icon
            ldbobj.text = questItem.description

            dqButton:Enable()
            if( chardb.Button.AutoHideButton ) then
                dqButton:ShowEx()
            end--if
        end--if
    end--if
    self.lastItem = questItem
end--Baldrick:SetButton( questItem )
---UI Functions--
-----------------
