--[[
    Baldrick_DailyQuests by LordFarlander
    Version 1.6.137
]]--

--[[
Copyright (c) 2012-2012, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 136 $
-- $Date: 2012-04-08 18:38:32 +0000 (Sun, 08 Apr 2012) $

local Baldrick = _G["Baldrick"]

if( not Baldrick ) then
    error( L["%s requires %s"]:format( "Baldrick_BurningCrusadeDailyQuests", "Baldrick" ) )
    return
end--if

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Baldrick_BurningCrusadeDailyQuests" )
local SpecialEventsAura = LibStub( "LibLordFarlander-SpecialEvents-Aura-4.0" )
local LBSZ = LibStub( "LibBabble-SubZone-3.0" )

if( LBSZ ) then
    LBSZ = LBSZ:GetUnstrictLookupTable()
end--if

local ZONES = {}
setmetatable( ZONES, {
    __index = function( self, key )
            return LBSZ and LBSZ[key] or L[key]
        end } )

local QuestObjectiveComplete = Baldrick.QuestObjectiveComplete

local function IsPlayerMountedOrBird()
   return IsMounted() or
          ( ( select( 2, UnitClass( "player" ) ) == "Druid" ) and
           ( select( GetShapeshiftFormInfo( 1, GetShapeshiftForm( true ) ) ):find( "Ability_Druid_FlightForm", 1, true ) ~= nil ) )
end--IsPlayerMountedOrBird()

local function IsTargetAliveAndNotTapped()
    return not( UnitIsDead( "playertarget" ) ) and
           ( UnitIsTappedByPlayer( "playertarget" ) or not( UnitIsTapped( "playertarget" ) ) )
end--IsTargetAliveAndNotTapped()

local burningCrusadeDailyQuests = {
    --[[
        These are the availiable fields; required are marked with a *, optional fields must be nil:
        questIDs: *Either a number with a questID or a table of numbers with questIDs.
        questItems: *A table of items required for the quest.  The one with an index of default is what will show on the button
                    The fields for items are:
                        questItem: the itemID of the item
                        questItemCount: the MINIMUM number required (the comparison will be >= )
        zones: A zone name this quest is valid for, ot a table of zone names
        subZones: A subzone name this quest is valid for, ot a table of zone names
        target: NPCId of target to check for, or table of NPCIds for multiple targets
        addionalEvents: Either a string with the name of an additional event that this quest should be checked when fired,
                        or a table of strings with names.  Events from a library that uses CallBackHandler can be registered for
                        buy using LibraryName:Event syntax.
        func: A function that takes two parameters.  The first parameter is the quest data iself, the second parameter will hold
              the questID being used.  If used this function must return a boolean, true if this quest is valid to show on the button.
        onEVENT: A function that will be ran whenever the event is encountered and the quest is registered for it.
        ]]--
    --Shattered Sun
    --Shattrath
    {
        --The Multiphase Survey
        questIDs = 11880,
        --Multiphase Spectrographic Goggles
        questItems = { default = { questItem = 35233, }, },
        zones = ZONES["Nagrand"],
        subZones = { ZONES["Spirit Fields"], "", },
        additionalEvents = { "ClosetGnome_WearSet", "ClosetGnome_PartlyWearSet", },
        onClosetGnome_WearSet = "CloseGnomeFunc",
        onClosetGnome_PartlyWearSet = "CloseGnomeFunc",
        onQuest_Complete = "QuestFunc",
        onQuest_Fail = "QuestFunc",
        onQuest_Lost = "QuestFunc",
        onZONE_CHANGED = "ZoneFunc",
        onZONE_CHANGED_NEW_AREA = "ZoneFunc",
        -- Nonstandard stuffs
        lastSet = "",
        CloseGnomeFunc =
            function( quest, set )
                if( set ~= Baldrick.GetQuestName( quest.questIDs ) ) then
                    quest.lastSet = set
                end--if
            end,
        QuestFunc =
            function( quest, questName, uid )
                if( ClosetGnome and ( uid == quest.questIDs ) and questName and
                    ClosetGnome:HasSet( questName ) and ClosetGnome:IsSetFullyEquipped( questName ) ) then
                    ClosetGnome:WearSet( quest.lastSet )
                end--if
            end,
        ZoneFunc =
            function( quest )
                local questID = quest.questIDs
                local questName = Baldrick.GetQuestName( questID )

                if( ClosetGnome and questName and Baldrick.HasQuestAndNotComplete( questID ) and ClosetGnome:HasSet( questName ) ) then
                    local subZone = GetSubZoneText()

                    if( ( subZone == ZONES["Spirit Fields"] ) or ( ( subZone == "" ) and ( quest.zones == GetRealZoneText() ) ) ) then
                        ClosetGnome:WearSet( questName )
                    else
                        ClosetGnome:WearSet( quest.lastSet )
                    end--if
                end--if
            end,
    },
    {
        --Intercepting the Mana Cells
        --Maintaining the Sunwell Portal
        questIDs = { 11513, 11514, },
        --Bash'ir Phasing Device
        questItems = { default = { questItem = 34248, }, },
        zones = ZONES["Blade's Edge Mountains"],
        subZones = ZONES["Bash'ir Landing"],
        additionalEvents = { "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffGained",
                             "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffLost"
                           },
        func =
            function( quest )
                return not SpecialEventsAura:UnitHasBuff( "player", select( 1, GetItemInfo( quest.questItems.default.questItem ) ) )
            end,
    },
    --Quel'Danas
    {
        --Erratic Behavior
        --Further Conversions
        questIDs = { 11524, 11525, },
        --Attuned Crystal Cores
        questItems = { default = { questItem = 34368, }, },
        --Erratic Sentry
        targets = 24972,
        func =
            function()
                return UnitIsDead( "playertarget" ) and
                       ( UnitIsTappedByPlayer( "playertarget" ) or not( UnitIsTapped( "playertarget" ) ) )
            end,
    },
    {
        --The Sanctum Wards
        --Arm the Wards!
        questIDs = { 11496, 11523, },
        --Mana Remnants
        questItems = { default = { questItem = 34338, questItemCount = 4, }, },
        zones = ZONES["Isle of Quel'Danas"],
        subZones = ZONES["Sun's Reach Sanctum"],
    },
    {
        --The Battle for the Sun's Reach Armory
        --The Battle Must Go On
        questIDs = { 11538, 11537, },
        --Shattered Sun Banner
        questItems = { default = { questItem = 34414, }, },
        --Emissary of Hate
        targets = 25003,
        func =
            function( _, questID )
                return UnitIsDead( "playertarget" ) and not( QuestObjectiveComplete( questID, 1 ) )
            end,
    },
    {
        --Distraction at the Dead Scar
        --The Air Strikes Must Continue
        questIDs = { 11532, 11533, },
        --Arcane Charges
        questItems = { default = { questItem = 34475, }, },
        zones = ZONES["Isle of Quel'Danas"],
        subZones = ZONES["The Dead Scar"],
        additionalEvents = { "PLAYER_CONTROL_LOST", "PLAYER_CONTROL_GAINED", "TAXIMAP_CLOSED" },
        func = function() return UnitOnTaxi( "player" ); end,
    },
    {
        --Disrupt the Greengill Coast
        questIDs = 11541,
        --Orb of Murloc Control
        questItems = { default = { questItem = 34483, }, },
        --Greengill Slave
        targets = 25084,
        func = IsTargetAliveAndNotTapped,
    },
    {
        --Know Your Ley Lines
        questIDs = 11547,
        --Astromancer's Crystal
        questItems = { default = { questItem = 34533, }, },
        zones = ZONES["Isle of Quel'Danas"],
        subZones = { ZONES["The Dawning Square"], ZONES["Dawnstar Village"], ZONES["Greengill Coast"], },
        func = function( quest, questid, _, subzone )
                   local objectives = quest.objectives

                   for index, name in ipairs( quest.subZones ) do
                       if( ( name == subzone ) and QuestObjectiveComplete( questid, objectives[index] ) ) then
                           return false
                       end--if
                   end--for
                   return true
               end,
        --nonstandard
        objectives = { 1, 2, 3, },
    },
    {
        --Intercept the Reinforcements
        --Keeping the Enemy at Bay
        questIDs = { 11542, 11543, },
        --Flaming Oil
        questItems = { default = { questItem = 34489, }, },
        additionalEvents = { "PLAYER_CONTROL_LOST", "PLAYER_CONTROL_GAINED", "TAXIMAP_CLOSED", "ZONE_CHANGED" },
        func =
            function( _, questid )
                return UnitOnTaxi( "player" ) and
                       ( not( QuestObjectiveComplete( questid, 1 ) ) or
                         not( QuestObjectiveComplete( questid, 2 ) ) or
                         not( QuestObjectiveComplete( questid, 3 ) ) )
            end,
    },
    {
        --Ata'mal Armaments
        questIDs = 11544,
        --Ata'mal Armament
        questItems = { default = { questItem = 34500, }, },
        zones = ZONES["Isle of Quel'Danas"],
        subZones = ZONES["Sun's Reach Armory"],
    },
    {
        --Discovering Your Roots
        --Rediscovering Your Roots
        questIDs = { 11520, 11521, },
        --Razorthorn Flayer Gland
        questItems = { default = { questItem = 34255, }, },
        --Razorthorn Ravager
        targets = 24922,
        func = IsTargetAliveAndNotTapped,
    },
    --Throne of Kil'jaeden
    {
        --Blood for Blood
        questIDs = 11515,
        --Fel Siphon, Demonic Blood
        questItems = { default = { questItem = 34257, }, { questItem = 34259, }, },
        --Felblood Initiate
        targets = 24918,
        func = IsTargetAliveAndNotTapped,
    },
    {
        --Blast the Gateway
        questIDs = 11516,
        --Sizzling Embers
        questItems = { default = { questItem = 34253, }, },
        zones = ZONES["Hellfire Peninsula"],
        subZones = ZONES["Throne of Kil'jaeden"],
    },

    --Skyguard
    --Skettis
    {
        --Fires Over Skettis
        questIDs = 11008,
        --Skyguard Blasting Charges
        questItems = { default = { questItem = 32406, }, },
        zones = ZONES["Terokkar Forest"],
        subZones = { ZONES["Lower Veil Shil'ak"],
                     ZONES["Skettis"],
                     ZONES["Terokk's Rest"],
                     ZONES["Upper Veil Shil'ak"],
                     ZONES["Veil Ala'rak"],
                     ZONES["Veil Harr'ik"],
                   },
        additionalEvents = { "LibLordFarlander-SpecialEvents-Mount-4.0:Mounted",
                             "LibLordFarlander-SpecialEvents-Mount-4.0:Dismounted",
                             "UPDATE_SHAPESHIFT_FORM"
                           },
        func = IsPlayerMountedOrBird,
    },
    --Blade's Edge Mountains
    {
        --Bombing Run
        --Bomb Them Again!
        questIDs = { 11010, 11102, 11023, },
        --Skyguard Bombs
        questItems = { default = { questItem = 32456, }, },
        zones = ZONES["Blade's Edge Mountains"],
        subZones = { ZONES["Forge Camp: Terror"], ZONES["Forge Camp: Wrath"], },
        additionalEvents = { "LibLordFarlander-SpecialEvents-Mount-4.0:Mounted",
                             "LibLordFarlander-SpecialEvents-Mount-4.0:Dismounted",
                             "UPDATE_SHAPESHIFT_FORM"
                           },
        func = IsPlayerMountedOrBird,
    },
    {
        --Wrangle Some Aether Rays!
        --Wrangle More Aether Rays!
        questIDs = { 11065, 11066, },
        --Wrangling Rope
        questItems = { default = { questItem = 32698, }, },
        --Aether Ray
        targets = 22181,
        func = IsTargetAliveAndNotTapped,
    },

    --Ogri'la
    {
        --Banish The Demons
        --Banish More Demons
        questIDs = { 11026, 11051, },
        --Banishing Crystal, Apexis Shard
        questItems = { default = { questItem = 32696, }, { questItem = 32569, }, },
        zones = ZONES["Blade's Edge Mountains"],
        subZones = { ZONES["Forge Camp: Terror"], ZONES["Forge Camp: Wrath"], },
    },

    --Netherwing
    {
        --The Booterang: A Cure For The Common Worthless Peon
        questIDs = 11055,
        --Booterang
        questItems = { default = { questItem = 32680, }, },
        --Disobedient Dragonmaw Peon
        targets = 23311,
        func = IsTargetAliveAndNotTapped,
    },
    {
        --A Slow Death
        questIDs = 11020,
        --Yarzill's Mutton
        --Fel Gland
        questItems = { default = { questItem = 32503, }, { questItem = 32502, }, },
        zones = ZONES["Netherstorm"],
        subZones = ZONES["Netherwing Ledge"],
    },
}

Baldrick:RegisterQuestModule( L["Burning Crusade Daily Quests"], burningCrusadeDailyQuests, nil, 2 )
