--[[
    Baldrick_DailyQuests by LordFarlander
    Version 1.6.137
]]--

--[[
Copyright (c) 2012, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 136 $
-- $Date: 2012-04-08 18:38:32 +0000 (Sun, 08 Apr 2012) $

local Baldrick = _G["Baldrick"]

if( not Baldrick ) then
    error( L["%s requires %s"]:format( "Baldrick_CataclysmDailyQuests", "Baldrick" ) )
    return
end--if

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Baldrick_CataclysmDailyQuests" )
local LBCT = LibStub( "LibBabble-CreatureType-3.0" ):GetLookupTable()
local LBSZ = LibStub( "LibBabble-SubZone-3.0" )

if( LBSZ ) then
    LBSZ = LBSZ:GetUnstrictLookupTable()
end--if

local ZONES = {}
setmetatable( ZONES, {
    __index = function( self, key )
            return LBSZ and LBSZ[key] or L[key]
        end } )

local cataclysmDailyQuests = {
    --[[
        These are the availiable fields; required are marked with a *, optional fields must be nil:
        questIDs: *Either a number with a questID or a table of numbers with questIDs.
        questItems: *A table of items required for the quest.  The one with an index of default is what will show on the button
                    The fields for items are:
                        questItem: the itemID of the item
                        questItemCount: the MINIMUM number required (the comparison will be >= )
        zones: A zone name this quest is valid for, ot a table of zone names
        subZones: A subzone name this quest is valid for, ot a table of zone names
        target: NPCId of target to check for, or table of NPCIds for multiple targets
        addionalEvents: Either a string with the name of an additional event that this quest should be checked when fired,
                        or a table of strings with names.  Events from a library that uses CallBackHandler can be registered for
                        buy using LibraryName:Event syntax.
        func: A function that takes two parameters.  The first parameter is the quest data iself, the second parameter will hold
              the questID being used.  If used this function must return a boolean, true if this quest is valid to show on the button.
        onEVENT: A function that will be ran whenever the event is encountered and the quest is registered for it.
        ]]--
    --Mount Hyjal
    {
        --Nature's Blessing
        questIDs = 29162,
        --Emerald of Aessina
        questItems = { default = { questItem = 69232, }, },
        zones = ZONES["Mount Hyjal"],
    },
    {
        --Perfecting Your Howl
        questIDs = 69235,
        --Fang of the Wolf
        questItems = { default = { questItem = 69832, }, },
        zones = ZONES["Mount Hyjal"],
    },
    {
        --The Power of Malorne
        questIDs = 29126,
        --Guardian's Staff
        questItems = { default = { questItem = 68997, }, },
        zones = ZONES["Mount Hyjal"],
    },
    {
        --Wings Aflame
        questIDs = 29148,
        --Quill of the Bird-Queen
        questItems = { default = { questItem = 69212, }, },
        zones = ZONES["Mount Hyjal"],
    },
    --Molten Front
    {
        --A Bitter Pill
        questIDs = 29263,
        --The Bitter Pill
        questItems = { default = { questItem = 69759, }, },
        --Subterranean Magma Worm
        targets = 53112,
    },
    {
        --Burn Victims
        questIDs = 29138,
        --Enchanted Salve
        questItems = { default = { questItem = 69240, }, },
        --Wounded Hyjal Defender
        targets = 52834,
    },
    {
        --Bye Bye Burdy
        questIDs = 29297,
        --Burd Sticker
        questItems = { default = { questItem = 69832, }, },
        --Druid of the Flame
        targets = 52661,
    },
    {
        --How Hot
        questIDs = 29273,
        --Tholo's Thermometer
        questItems = { default = { questItem = 69806, }, },
        zones = ZONES["Molten Front"],
        subZones = ZONES["Fireplume Peak"],
    },
    -- Tol Barad
    {
        --Magnets, How Do They Work?
        questIDs = { 28692, 27992, },
        --Magnetized Scrap Collector
        questItems = { default = { questItem = 62829, }, },
        zones = ZONES["Tol Barad Peninsula"],
        subZones = ZONES["The Restless Front"],
    },
}

Baldrick:RegisterQuestModule( L["Cataclysm Daily Quests"], cataclysmDailyQuests, nil, 2 )
