--[[
    Baldrick_DailyQuests by LordFarlander
    Version 1.6.137
]]--

--[[
Copyright (c) 2012-2012, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 136 $
-- $Date: 2012-04-08 18:38:32 +0000 (Sun, 08 Apr 2012) $

local Baldrick = _G["Baldrick"]

if( not Baldrick ) then
    error( L["%s requires %s"]:format( "Baldrick_HolidayDailyQuests", "Baldrick" ) )
    return
end--if

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Baldrick_HolidayDailyQuests" )
local SpecialEventsAura = LibStub( "LibLordFarlander-SpecialEvents-Aura-4.0" )
local LBSZ = LibStub( "LibBabble-SubZone-3.0" )

if( LBSZ ) then
    LBSZ = LBSZ:GetUnstrictLookupTable()
end--if

local ZONES = {}
setmetatable( ZONES, {
    __index = function( self, key )
            return LBSZ and LBSZ[key] or L[key]
        end } )

local HOLIDAYS = L

local holidayCache = {}
local lastDate = {}

local function IsHolidayToday( quest )
    local holiday = quest.holiday
    local _, month, day, year = CalendarGetDate()

    if( not( lastDate.month ) or ( lastDate.day ~= day ) or ( lastDate.month ~= month ) or ( lastDate.year ~= year ) ) then
        -- Rebuild the cache
        lastDate.day = day
        lastDate.month = month
        lastDate.year = year
        table.wipe( holidayCache )
        for i = 1, CalendarGetNumDayEvents( 0, day ) do
            local title, hour, minute, calendarType, sequenceType = CalendarGetDayEvent( 0, day, i )

            if( calendarType == "HOLIDAY" ) then
                local gameHour, gameMinute = GetGameTime()

                if( sequenceType == "START" ) then
                    -- The holiday starts today, check if now is after the start
                    holidayCache[title] = ( gameHour > hour ) or ( ( gameHour == hour ) and ( gameMinute >= minute ) )
                elseif( sequenceType == "END" ) then
                    -- The holiday ends today, check if now is before the end
                    holidayCache[title] = ( gameHour < hour ) or ( ( gameHour == hour ) and ( gameMinute < minute ) )
                else
                    -- in the middle of the holiday, it's definitely on
                    holidayCache[title] = true
                end--if
            end--if
        end--for
    end--if
    if( holidayCache[holiday] ) then
        return true
    end--if
    -- The holiday wasn't found for today
    return false
end--IsHolidayToday( holiday )

local holidayDailyQuests = {
    --[[
        These are the availiable fields; required are marked with a *, optional fields must be nil:
        questIDs: *Either a number with a questID or a table of numbers with questIDs.
        questItems: *A table of items required for the quest.  The one with an index of default is what will show on the button
                    The fields for items are:
                        questItem: the itemID of the item
                        questItemCount: the MINIMUM number required (the comparison will be >= )
        zones: A zone name this quest is valid for, ot a table of zone names
        subZones: A subzone name this quest is valid for, ot a table of zone names
        target: NPCId of target to check for, or table of NPCIds for multiple targets
        addionalEvents: Either a string with the name of an additional event that this quest should be checked when fired,
                        or a table of strings with names.  Events from a library that uses CallBackHandler can be registered for
                        buy using LibraryName:Event syntax.
        func: A function that takes two parameters.  The first parameter is the quest data iself, the second parameter will hold
              the questID being used.  If used this function must return a boolean, true if this quest is valid to show on the button.
        onEVENT: A function that will be ran whenever the event is encountered and the quest is registered for it.
        ]]--
    --Brewfest
    {
        --Now This is Ram Racing... Almost.
        --There and Back Again
        --Bark for Drohn's Distillery!
        --Bark for T'chali's Voodoo Brewery!
        --Bark for the Barleybrews!
        --Bark for the Thunderbrews!
        questIDs = { 11318, 11409, 11122, 11412, 11407, 11408, 11293, 11294, },
        --Ram Racing Reins
        questItems = { default = { questItem = 33306, }, },
        additionalEvents = { "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffGained", "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffLost" },
        func =
            function( quest )
                return IsHolidayToday( quest ) and SpecialEventsAura:UnitHasBuff( "player", quest.buff )
            end,
        --nonstandard
        --Rental Racing Ram
        buff = 43883,
        holiday = HOLIDAYS["Brewfest"],
    },
    --Hallow's End
    {
        --A Time to Break Down
        questIDs = 29318,
        --Dousing Agent
        questItems = { default = { questItem = 68647, }, },
        zones = ZONES["Elwynn Forest"],
        func = IsHolidayToday,
        --nonstandard
        holiday = HOLIDAYS["Hallow's End"],
    },
    {
        --A Time to Lose
        questIDs = 29318,
        --Dousing Agent
        questItems = { default = { questItem = 68648, }, },
        zones = ZONES["Ruins of Lordaeron"],
        func = IsHolidayToday,
        --nonstandard
        holiday = HOLIDAYS["Hallow's End"],
    },
    {
        --Clean Up in Stormwind
        questIDs = 29144,
        --Arcane Cleanser
        questItems = { default = { questItem = 69191, }, },
        zones = ZONES["Stormwind"],
        func = IsHolidayToday,
        --nonstandard
        holiday = HOLIDAYS["Hallow's End"],
    },
    {
        --Clean Up in Undercity
        questIDs = 29375,
        --Arcane Cleanser
        questItems = { default = { questItem = 70727, }, },
        zones = ZONES["Undercity"],
        func = IsHolidayToday,
        --nonstandard
        holiday = HOLIDAYS["Hallow's End"],
    },
    {
        --Fire Brigade Practice
        --Fire Training
        --Stop the Fires!
        --"Let the Fires Come!"
        questIDs = { 11360, 11439, 11440, 11361, 11449, 11450, 11131, 11219, 12135, 12139, },
        --Water Bucket
        questItems = { default = { questItem = 32971, }, },
        func = IsHolidayToday,
        --nonstandard
        holiday = HOLIDAYS["Hallow's End"],
    },
    --Love is in the Air
    {
        --Crushing the Crown
        questIDs = { 24638, 24645, 24647, 24648, 24658, 24649, 24650, 24651, 24652, 24659,
                     24660, 24662, 24663, 24664, 24665, 24666, 28934, 28935,
                   },
        --Snagglebolt's Khorium Bomb
        questItems = { default = { questItem = 70727, }, },
        --Chemical Wagon
        targets = 38035,
        func = IsHolidayToday,
        --nonstandard
        holiday = HOLIDAYS["Love is in the Air"],
    },
    --Noblegarden
    {
        --The Great Egg Hunt
        questIDs = { 13479, 13480, },
        --Brightly Colored Egg
        questItems = { default = { questItem = 45072, questItemCount = 1, } },
        func = IsHolidayToday,
        --nonstandard
        holiday = HOLIDAYS["Noblegarden"],
    },
    --Midsummer Fire Festival
    {
        --Torch Catching
        --More Torch Catching
        questIDs = { 11657, 11923, 11924, 11925, },
        --Unlit Torches
        questItems = { default = { questItem = 34833, }, },
        func = IsHolidayToday,
        --nonstandard
        holiday = HOLIDAYS["Midsummer Fire Festival"],
    },
    {
        --Torch Tossing
        --More Torch Tossing
        questIDs = { 11731, 11922, 11921, 11926, },
        --Practice Torches
        questItems = { default = { questItem = 34862, }, },
        func = IsHolidayToday,
        --nonstandard
        holiday = HOLIDAYS["Midsummer Fire Festival"],
    },
}

Baldrick:RegisterQuestModule( L["Holiday Daily Quests"], holidayDailyQuests, nil, 2 )
